#include <stdio.h>
#include <stdlib.h>

#ifndef __APPLE__
#include <GL/glut.h>
#else
#include <GLUT/glut.h>
#endif

struct point {
	float x, y;
};

void disp(void);
int sierp_tri(struct point p1, struct point p2, struct point p3, int subdiv);
struct point midpoint(struct point a, struct point b);
void reshape(int x, int y);
void keyb(unsigned char key, int x, int y);
void mouse(int bn, int state, int x, int y);
void motion(int x, int y);

struct point pt[3] = {
	{400, 100},
	{600, 500},
	{200, 500}
};
int subdiv = 0;

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(800, 600);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_MULTISAMPLE);
	glutCreateWindow("sierpinski");

	glutDisplayFunc(disp);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyb);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);

	glEnable(GL_MULTISAMPLE);

	glutMainLoop();
	return 0;
}


void disp(void)
{
	int ntri;
	char buf[128];

	glClear(GL_COLOR_BUFFER_BIT);

	glBegin(GL_TRIANGLES);
	ntri = sierp_tri(pt[0], pt[1], pt[2], subdiv);
	glEnd();

	sprintf(buf, "sierpinski(%d) -> %d trigangles", subdiv, ntri);
	glutSetWindowTitle(buf);

	glutSwapBuffers();
}

int sierp_tri(struct point p1, struct point p2, struct point p3, int subdiv)
{
	int ntri;
	struct point p12, p31, p23;

	if(subdiv <= 0) {
		glVertex2f(p1.x, p1.y);
		glVertex2f(p2.x, p2.y);
		glVertex2f(p3.x, p3.y);
		return 1;
	}

	p12 = midpoint(p1, p2);
	p23 = midpoint(p2, p3);
	p31 = midpoint(p3, p1);

	ntri = sierp_tri(p1, p12, p31, subdiv - 1);
	ntri += sierp_tri(p12, p2, p23, subdiv - 1);
	ntri += sierp_tri(p31, p23, p3, subdiv - 1);
	return ntri;
}

struct point midpoint(struct point a, struct point b)
{
	struct point res;
	res.x = (a.x + b.x) * 0.5;
	res.y = (a.y + b.y) * 0.5;
	return res;
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, x, y, 0, -1, 1);
}

void keyb(unsigned char key, int x, int y)
{
	static int fullscr;

	switch(key) {
	case 27:
		exit(0);

	case 'f':
		fullscr = !fullscr;
		if(fullscr) {
			glutFullScreen();
		} else {
			glutReshapeWindow(800, 600);
		}
		glutPostRedisplay();
		break;

	case '=':
		subdiv++;
		printf("subdiv: %d\n", subdiv);
		glutPostRedisplay();
		break;

	case '-':
		if(subdiv > 0) {
			subdiv--;
			printf("subdiv: %d\n", subdiv);
			glutPostRedisplay();
		}
		break;
	}
}

static int sel_idx = -1;

void mouse(int bn, int state, int x, int y)
{
	if(state == GLUT_DOWN) {
		int i;
		for(i=0; i<4; i++) {
			if(abs(pt[i].x - x) < 5 && abs(pt[i].y - y) < 5) {
				sel_idx = i;
				break;
			}
		}
	} else {
		sel_idx = -1;
	}
}

void motion(int x, int y)
{
	if(sel_idx != -1) {
		pt[sel_idx].x = x;
		pt[sel_idx].y = y;
		glutPostRedisplay();
	}
}
